/*
Murasaki - multiple genome global alignment program
Copyright (C) 2006-2007 Keio University
(Yasunori Osana) <comp@bio.keio.ac.jp> (2006)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 3
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "sequence.h"

#ifndef _DBHASH_H_
#define _DBHASH_H_

#ifdef MURASAKI_MYSQL
class MySQLHash : public Hash {
 public:
  void clear();
  void add(const HashKey key,const HashVal &val);
  void lookup(HashKey key,const HashVal &a,LocList &locList);
  void lookup(HashKey key,LocList &locList);
  bool emptyAt(const HashKey key);
  word sizeAt(const HashKey key);

  MySQLHash(BitSequence *pat);
  ~MySQLHash();

  static const word linear_cost(word c)      { return 0; }
  static const word bucket_prep_cost(word c) { return 0; }

  // now sanitycheck yet. soooooory.
  bool rawSanityCheck();
  bool rawSanityCheck(word base);

 protected:
  MYSQL *fasta; 
  bool indexed;

  MYSQL_STMT    *stmt_insert, *stmt_select;
  MYSQL_BIND    bind_insert[3], bind_select[1], bind_selectr[3];
  MYSQL_RES     *res_select;

  int64_t insert_key, insert_pos, select_key;
  int64_t selectr_key, selectr_pos;
  int32_t insert_seq; 
  int32_t selectr_seq;
  
  unsigned long selectr_len[3];
  my_bool       selectr_nul[3];
  my_bool       selectr_err[3];

  char *insert_SQL;
  char *select_SQL;

  void createIndex();
};
#endif

#ifdef MURASAKI_SQLITE3
class SQLiteHash : public Hash {
 public:
  void clear();
  void add(const HashKey key,const HashVal &val);
  void lookup(HashKey key,const HashVal &a,LocList &locList);
  void lookup(HashKey key,LocList &locList);
  bool emptyAt(const HashKey key);
  word sizeAt(const HashKey key);

  SQLiteHash(BitSequence *pat);
  ~SQLiteHash();

  static const word linear_cost(word c)      { return 0; }
  static const word bucket_prep_cost(word c) { return 0; }

  // now sanitycheck yet. soooooory.
  bool rawSanityCheck();
  bool rawSanityCheck(word base);

 protected:
  sqlite3 *fasta; 
  bool indexed;
  
  void createIndex();
};
#endif

#endif
